@@ -2,11 +2,10 @@ mod cli;
 mod env;
 mod yaml;
 
+use crate::commands::func::Func;
 use crate::finder::FinderChoice;
-
-use crate::terminal::style::Color;
 pub use cli::*;
-
+use crossterm::style::Color;
 use env::EnvConfig;
 use yaml::YamlConfig;
 
@@ -23,7 +22,7 @@ impl Config {
     pub fn new() -> Self {
         let env = EnvConfig::new();
         let yaml = YamlConfig::get(&env).unwrap_or_else(|e| {
-            eprintln!("Error parsing config file: {}", e);
+            eprintln!("Error parsing config file: {e}");
             eprintln!("Fallbacking to default one...");
             eprintln!();
             YamlConfig::default()
@@ -36,6 +35,10 @@ impl Config {
         self.clap.best_match
     }
 
+    pub fn prevent_interpolation(&self) -> bool {
+        self.clap.prevent_interpolation
+    }
+
     pub fn cmd(&self) -> Option<&Command> {
         self.clap.cmd.as_ref()
     }
@@ -45,8 +48,14 @@ impl Config {
             Source::Tldr(query)
         } else if let Some(query) = self.clap.cheatsh.clone() {
             Source::Cheats(query)
+        } else if let Some(Command::Fn(input)) = self.cmd() {
+            if let Func::Welcome = input.func {
+                Source::Welcome
+            } else {
+                Source::Filesystem(self.path())
+            }
         } else {
-            Source::Filesystem(self.path(), self.tag_rules())
+            Source::Filesystem(self.path())
         }
     }
 
